from lib.snabbdom import h, patch
from message import *
import random

from export import export_to_xml


class View:
    def __init__(self, container):
        self.vdom = h('div', {}, "Loading...")
        self.model = None
        patch(container, self.vdom)
    
    def view(self, model):
        self.model = model
        new_vdom = self._view()
        patch(self.vdom, new_vdom)
        self.vdom = new_vdom
    
    def _view(self):
        return h("div", {"on": { "click": msg(Reset) }}, [
          View.view_entry(self.model.entry),
          h("button.blk", {"on": { "click": lambda _: console.log(export_to_xml(self.model)) } }, "XML2Console"),
          View.view_menu(self.model.menu_location, self.model.menu_shown, self.model.translation),
          View.view_modal(self.model.modal_shown, self.model.modal)])
       
    @staticmethod
    def view_entry(entry):
        view_sense_list = [View.view_sense(sense, idx) for idx, sense in enumerate(entry.senses)]
        
        return h("div#entry", {}, [
          h("div#entry-status", {}, entry.status),
          h("div#entry-header", {}, [
            h("span#headword", {}, entry.headword),
            h("span#grammar", {}, entry.grammar),
            h("button#comment.warning", {"on": {"click": msg(ShowCommentEdit)}}, entry.comment)]),
          h("div#sense-container", {}, view_sense_list)])
    
    @staticmethod
    def view_sense(sense, senseNum):
        examples = [View.view_example(example) for example in sense.examples]
        
        return h("div.elm-div", {}, [
          h("div.sense-num", {}, str(senseNum + 1)),
          h("div.sense", {}, [
            h("span.sense-label-list", { "on": { "click": msg(ShowSenseLabelEdit, sense) }}, [
                h("span.sense-label", {}, slabel) for slabel in sense.labels ]), 
            h("span.sense-definition", { "on": { "click": msg(ShowSenseDefinitionEdit, sense) }}, sense.definition),
            h("div", {}, View.view_translations(sense.translations, sense)),
            h("div", {}, examples)])])
    
    @staticmethod
    def view_example(example):
        return h("div.example", {}, [
          h("div.example-dot", {}, "▣"),
          h("div.example-rest", {}, [
            h("span.example-text", {}, example.example),
            h("div.example-translation", {}, [
              h("span.example-arrow", {}, "↪"),
              h("span", {}, example.translation)])])])
    
    @staticmethod
    def view_translations(translations, sense):
        joiner = lambda: h("span.translation-semicolon", {}, ";")
        result = []
        
        for cluster in translations:
            result.extend([View.view_one_translation(t) for t in cluster])
            result.append(joiner())
        
        # remove last ';' and add + button; [-1] does not work in transcrypt
        result[len(result) - 1] = h("button", {"on": {"click": msg(ShowAddTranslation, sense)}}, "+")
        return result
    
    @staticmethod
    def view_one_translation(translation):
        elements = []
    
        if translation.tags:
            tags = h("div.translation-tags", {}, [
              h("span", {"attr": {"title": key}}, value) 
              for key, value in translation.tags.items()])
            elements.append(tags)
        
        elements.append(h("span.translation-text", {}, translation.translation))
        return h("div.translation-div", {"on": {"click": msg(ShowMenu, translation) }}, elements)

    
    @staticmethod
    def view_menu(location, menu_shown, translation): 
        style = {
            "left": str(location[0]),
            "top": str(location[1])
        }
        if menu_shown:
            style["opacity"] = "1"
            style["visibility"] = "visible"
        
        return h("span.popup-menu", { "style": style }, [
            h("button.shyButton", { "on": {"click": msg(ShowEditTranslation, translation)}}, "✎"),
            h("button.shyButton", { "on": {"click": msg(MoveRight, translation)}}, "→"),
            h("button.shyButton", { "on": {"click": msg(MoveLeft, translation)}}, "←"),
            h("button.shyButton", { "on": {"click": msg(BinTranslation, translation)}}, "🗑")])
        
        
    @staticmethod
    def view_modal(modal_shown, modal):
        return h("div.modal", {}, [
            h("input", { "props": {"type": "checkbox", "checked": modal_shown} }, ""), 
            h("label.overlay", {}, ""), 
            h("article", {"on": { "click": NoReset }}, modal())])

